@@ -42,11 +42,12 @@
 
 typedef struct vm_s {
   x86emu_t *emu;
-  unsigned char *video_mem;
+  x86emu_memio_handler_t old_memio;
 
   unsigned ports;
   unsigned force:1;
   unsigned timeout;
+  unsigned no_io:1;
 
   unsigned all_modes:1;
   unsigned mode;
@@ -78,6 +79,7 @@ static void vm_free(vm_t *vm);
 static unsigned vm_run(x86emu_t *emu, double *t);
 static int vm_prepare(vm_t *vm);
 
+static unsigned new_memio(x86emu_t *emu, u32 addr, u32 *val, unsigned type);
 static double get_time(void);
 static void *map_mem(vm_t *vm, unsigned start, unsigned size, int rw);
 
@@ -178,6 +180,8 @@ void get_vbe_info(hd_data_t *hd_data, vbe_info_t *vbe)
   if(hd_probe_feature(hd_data, pr_bios_fb)) {
     PROGRESS(4, 2, "mode info");
 
+    // there shouldn't any real io be needed for this
+    vm->no_io = 1;
     list_modes(vm, vbe);
   }
 
@@ -186,12 +190,16 @@ void get_vbe_info(hd_data_t *hd_data, vbe_info_t *vbe)
 
     ADD2LOG("vbe: probing %d ports\n", vm->ports);
 
+    // for ddc probing we have to allow direct io accesses
+    vm->no_io = 0;
     probe_all(vm, vbe);
   }
 
   if(hd_probe_feature(hd_data, pr_bios_mode)) {
     PROGRESS(4, 4, "gfx mode");
 
+    // there shouldn't any real io be needed for this
+    vm->no_io = 1;
     get_video_mode(vm, vbe);
   }
 
@@ -269,8 +277,8 @@ int do_int(x86emu_t *emu, u8 num, unsigned type)
     return 0;
   }
 
-  // ignore ints != 0x10
-  if(num != 0x10) return 1;
+  // ignore ints != (0x10 or 0x42 or 0x6d)
+  if(num != 0x10 && num != 0x42 && num != 0x6d) return 1;
 
   return 0;
 }
@@ -310,7 +318,7 @@ unsigned vm_run(x86emu_t *emu, double *t)
   vm_t *vm = emu->private;
   unsigned err;
 
-  x86emu_log(emu, "=== emulation log %d ===\n", vm->exec_count);
+  x86emu_log(emu, "=== emulation log %d %s===\n", vm->exec_count, vm->no_io ? "(no i/o) " : "");
 
   *t = get_time();
 
@@ -362,7 +370,9 @@ int vm_prepare(vm_t *vm)
     return ok;
   }
 
-  copy_to_vm(vm->emu, 0x10*4, p1 + 0x10*4, 4, X86EMU_PERM_RW);
+  copy_to_vm(vm->emu, 0x10*4, p1 + 0x10*4, 4, X86EMU_PERM_RW);		// video bios entry
+  copy_to_vm(vm->emu, 0x42*4, p1 + 0x42*4, 4, X86EMU_PERM_RW);		// old video bios entry
+  copy_to_vm(vm->emu, 0x6d*4, p1 + 0x6d*4, 4, X86EMU_PERM_RW);		// saved video bios entry
   copy_to_vm(vm->emu, 0x400, p1 + 0x400, 0x100, X86EMU_PERM_RW);
 
   munmap(p1, 0x1000);
@@ -384,14 +394,9 @@ int vm_prepare(vm_t *vm)
     x86emu_read_word(vm->emu, 0x10*4)
   );
 
-  // video memory
-  vm->video_mem = map_mem(vm, VBIOS_MEM, VBIOS_MEM_SIZE, 1);
-
-  if(vm->video_mem) {
-    x86emu_set_perm(vm->emu, VBIOS_MEM, VBIOS_MEM + VBIOS_MEM_SIZE - 1, X86EMU_PERM_RW);
-    for(u = 0; u < VBIOS_MEM_SIZE; u += X86EMU_PAGE_SIZE) {
-      x86emu_set_page(vm->emu, VBIOS_MEM + u, vm->video_mem + u);
-    }
+  // initialize fake video memory
+  for(u = VBIOS_MEM; u < VBIOS_MEM + VBIOS_MEM_SIZE; u++) {
+    vm_write_byte(vm->emu, u, 0, X86EMU_PERM_RW);
   }
 
   // start address 0:0x7c00
@@ -407,12 +412,37 @@ int vm_prepare(vm_t *vm)
 
   if(vm->timeout) vm->emu->timeout = vm->timeout ?: 20;
 
+  vm->old_memio = x86emu_set_memio_handler(vm->emu, new_memio);
+
   ok = 1;
 
   return ok;
 }
 
 
+/*
+ * Use our own memory and i/o access handler to block all i/o accesses if vm->no_io
+ * is set.
+ */
+unsigned new_memio(x86emu_t *emu, u32 addr, u32 *val, unsigned type)
+{
+  vm_t *vm = emu->private;
+
+  if(vm->no_io) {
+    if((type & ~0xff) == X86EMU_MEMIO_I) {
+      *val = 0;
+      return 0;
+    }
+
+    if((type & ~0xff) == X86EMU_MEMIO_O) {
+      return 0;
+    }
+  }
+
+  return vm->old_memio(emu, addr, val, type);
+}
+
+
 double get_time()
 {
   static struct timeval t0 = { };
